require("dotenv").config();
const fastify = require("fastify");
const Sentry = require("@sentry/node");

const mongo = require("./data/mongo/mongo");
const bodyParser = require("./middleware/bodyParser");
const requestHandler = require("./middleware/requestHandler");
const responseHandler = require("./middleware/responseHandler");
const errorHandler = require("./middleware/errorHandler");
const Logger = require("./middleware/logger");
const scooter = require("./routes/scooter");

const server = fastify();

server.addContentTypeParser(
	"application/json",
	{ parseAs: "string" },
	bodyParser,
);

server.register(require("fastify-compress"));

// Routes
server.register(scooter, { prefix: "/scooter" });

const logger = new Logger();

server.addHook("onRequest", requestHandler(logger));

server.addHook("onSend", responseHandler(logger));

server.setErrorHandler(errorHandler(logger));

const connectMongo = async () => {
	if (await mongo.connect(process.env.MONGO_CONNECTION)) {
		console.log("MongoDB connected");
	} else {
		throw new Error("Mongo connection failed!");
	}
};

const start = async () => {
	try {
		await server.listen(process.env.SERVER_PORT, "0.0.0.0");
		console.log("Server is listening on port: ", process.env.SERVER_PORT);

		await connectMongo();
	} catch (err) {
		console.log(err);
		Sentry.captureException(err);
		process.exit(1);
	}
};

if (require.main === module) {
	// called directly i.e. "node app"
	start();
} else {
	// required as a module => executed on aws lambda
	module.exports = server;
}
