const mongo = require("../mongo/mongo");
const ScooterDto = require("../../dto/scooterDto");

const ScooterSchema = new mongo.Schema({
	code: { type: String, index: { unique: true } },
	status_id: { type: Number, index: true },
	vehicle_type: { type: Number, index: true },
	battery_status: Number,
	last_update_time: Date,
	starting_price: Number,
	recurring_price: Number,
	starting_price_without_discount: Number,
	recurring_price_without_discount: Number,
	hdop: Number,
	reservation_price: Number,
	scooter_body_version_id: Number,
	geofence_group: Number,
	is_incentivized: Boolean,
	version: Number,
	is_available: Boolean,
	gsm_available: Boolean,
	last_known_point: {
		type: {
			type: String,
			enum: ["Point"],
			default: "Point",
		},
		coordinates: {
			type: [Number],
			default: [0, 0],
		},
	},
}, { versionKey: false, timestamps: true });

ScooterSchema.index({ last_known_point: "2dsphere" });

const ScooterModel = mongo.model("Scooter", ScooterSchema);

const modelConvertor = (scooter) => {
	const model = { ...scooter };

	model.last_known_point = {
		coordinates: [model.lon, model.lat],
	};

	const scooterModel = new ScooterModel(model);
	return scooterModel;
};

const domainConvertor = (model) => {
	if (!model) {
		return null;
	}

	return new ScooterDto(model.code,
		model.vehicle_type,
		model.status_id,
		model.battery_status,
		model.starting_price,
		model.recurring_price,
		model.last_update_time,
		model.last_known_point.coordinates[1],
		model.last_known_point.coordinates[0],
		model.starting_price_without_discount,
		model.recurring_price_without_discount,
		model.distance,
		model.hdop,
		model.reservation_price,
		model.is_incentivized,
		model.geofence_group);
};

const fetchScooters = mongo.fetch(ScooterModel, domainConvertor);

const removeScooters = mongo.remove(fetchScooters);

const fetchScooterById = mongo.fetchById(ScooterModel, domainConvertor);

const selector = (scooter) => ({
	scooterId: scooter.scooterId,
});

async function saveScooter(entities) {
	const saver = mongo.save(ScooterModel, modelConvertor, selector);
	return domainConvertor(await saver(entities));
}

const geoNear = mongo.geoNear(ScooterModel, domainConvertor);

class ScooterDataAccess {
	async fetch(query) {
		const result = await fetchScooters(mongo.convertIdInQuery(query));
		return result;
	}

	async fetchById(id) {
		const result = await fetchScooterById(id);
		return result && result.length > 0 ? result[0] : null;
	}

	async remove(query) {
		const result = await removeScooters(mongo.convertIdInQuery(query));
		return result;
	}

	async save(entities) {
		const result = await saveScooter(entities);
		return result;
	}

	async geoNear(lat, lon, query, maxDistance = 50000, limit = 20) {
		const result = await geoNear(lat, lon, query, maxDistance, limit);
		return result;
	}
}

module.exports = ScooterDataAccess;
