const mongoose = require("mongoose");
require("mongoose-long")(mongoose);
const { updateIfCurrentPlugin } = require("mongoose-update-if-current");

const {
	save, remove, fetch, fetchById, model, copyTo, convertIdInQuery, geoNear,
} = require("./mongohelper");

const mongoConnectionState = {
	disconnected: 0,
	connected: 1,
	connecting: 2,
	disconnecting: 3,
};

mongoose.set("useCreateIndex", true);

mongoose.plugin(updateIfCurrentPlugin, { strategy: "timestamp" });

const isConnected = (state) => state === mongoConnectionState.connected || state === mongoConnectionState.connecting;

const defaultConnectionOptions = {
	socketTimeoutMS: 30000,
	connectTimeoutMS: 10000,
	keepAlive: 1000,
	useNewUrlParser: true,
	useUnifiedTopology: true,
};

const connect = async (mongodbConnection, options = null) => {
	if (!isConnected(mongoose.connection.readyState)) {
		mongoose.Promise = require("bluebird");
		mongoose.set("useFindAndModify", false);
		await mongoose.connect(mongodbConnection, options || defaultConnectionOptions);
	}
	return isConnected(mongoose.connection.readyState);
};

const createConnection = async (mongodbConnection, name, options = null) => {
	mongoose.Promise = require("bluebird");
	mongoose.set("useFindAndModify", false);
	const dbOptions = options || defaultConnectionOptions;
	dbOptions.dbName = name;
	const connection = await mongoose.createConnection(mongodbConnection, dbOptions);
	return isConnected(connection.readyState);
};

const createDatabase = async (mongodbConnection, dbName, testCollection = "_tc_", options = null) => {
	const dbOptions = options || defaultConnectionOptions;
	dbOptions.dbName = dbName;
	const connection = await mongoose.createConnection(mongodbConnection, dbOptions);
	await connection.createCollection(testCollection);
	await connection.close();
};

const dropDatabase = async (mongodbConnection, dbName, options = null) => {
	const dbOptions = options || defaultConnectionOptions;
	dbOptions.dbName = dbName;
	const connection = await mongoose.createConnection(mongodbConnection, dbOptions);
	await connection.dropDatabase();
	await connection.close();
};

const stats = async (connection) => connection.db.stats({ scale: 1024 });

module.exports = {
	connect,
	mongoConnections: mongoose.connections,
	createConnection,
	createDatabase,
	dropDatabase,
	stats,
	save,
	remove,
	fetch,
	fetchById,
	copyTo,
	model: model(mongoose),
	multiConnectionModel: (connection) => model(connection),
	validateObjectId(id) {
		try {
			mongoose.Types.ObjectId(id);
			return true;
		} catch (ex) {
			return false;
		}
	},
	Schema: mongoose.Schema,
	disconnect: async () => mongoose.disconnect(),
	mongoConnectionState,
	convertIdInQuery,
	geoNear,
};
